@@ -4,7 +4,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = 0.1201;
+our $VERSION = 0.13;
 
 =head1 NAME
 
@@ -163,8 +163,9 @@ They have nothing to do with serving the request.
 =cut
 
 use Carp;
-use Scalar::Util qw(blessed);
+use Scalar::Util qw(blessed looks_like_number);
 use Encode;
+use POSIX qw(strftime);
 use parent qw(Exporter);
 
 our @EXPORT_OK = qw(neaf_err);
@@ -175,8 +176,8 @@ our $Inst = __PACKAGE__->new;
 
 =head2 route( path => CODEREF, %options )
 
-Creates a new route in the application.
-Any incoming request to uri starting with C</path>
+Set up an URI handler in the application.
+Any incoming request to uri matching C</path>
 (C</path/something/else> too, but NOT C</pathology>)
 will now be directed to CODEREF.
 
@@ -193,18 +194,39 @@ Exactly one leading slash will be prepended no matter what you do.
 
 =over
 
-=item * description - just for information, has no action on execution.
+=item * method - list of allowed HTTP methods.
+Default is [GET, POST, HEAD].
+Multiple handles can be defined for the same path, provided that
+metyhods do not intersect.
 
-=item * method - list of allowed HTTP methods. Default is [GET, POST, HEAD].
+=item * path_info_regex => qr/.../ - allow URI subpaths matching
+the specified expression to be handled by this handler.
+
+If specified, 404 error will be returned unless PATH_INFO matches the regex
+(without the leading slash).
+
+Starting from v.0.16, this parameter is going to be REQUIRED for path_info()
+method to be used in the handler.
+Until then, a DEPRECATED warning will be generated for a naked path_info call.
+
+B<EXPERIMENTAL>. Name and semantics MAY change in the future.
 
 =item * view - default View object for this Controller.
 Must be an object with a C<render> methods, or a CODEREF
 receiving hash and returning a list of two scalars.
 
+=item * cache_ttl - if set, set Expires: HTTP header accordingly.
+
+B<EXPERIMENTAL>. Name and semantics MAY change in the future.
+
+=item * description - just for information, has no action on execution.
+This will be displayed if application called with --list (see L<MVC::Neaf::CLI>).
+
 =back
 
 =cut
 
+my $year = 365 * 24 * 60 * 60;
 sub route {
     my $self = shift;
 
@@ -237,6 +259,13 @@ sub route {
     my %profile;
     $profile{code}     = $sub;
     $profile{caller}   = [caller(0)]; # file,line
+    if ( !defined $args{path_info_regex} ) {
+        # TODO replace def path_info_regex with '' in v.0.16 aka '404 if unspecified'
+        $args{path_info_regex}      = '.*';
+    } else {
+        $profile{has_path_info_regex} = 1;
+    };
+    $profile{path_info_regex_regex}  = qr#^/*($args{path_info_regex})$#;
 
     # Just for information
     $profile{path}        = $path;
@@ -255,13 +284,22 @@ sub route {
         $profile{view} = $view;
     };
 
+    if ( $args{cache_ttl} ) {
+        $self->_croak("cache_ttl must be a number")
+            unless looks_like_number($args{cache_ttl});
+        # as required by RFC
+        $args{cache_ttl} = -100000 if $args{cache_ttl} < 0;
+        $args{cache_ttl} = $year if $args{cache_ttl} > $year;
+        $profile{cache_ttl} = $args{cache_ttl};
+    };
+
     # ready, install handler & burn cache
     delete $self->{route_re};
     $self->{route}{ $path }{$_} = \%profile
         for @{ $args{method} };
 
     return $self;
-};
+}; # end sub route
 
 =head2 alias( $newpath => $oldpath )
 
@@ -315,8 +353,14 @@ Default is 4096. Smaller values may be set, but are NOT recommended.
 
 =item * cache_ttl => nnn - if given, files below the buffer size will be stored
 in memory for cache_ttl seconds.
+
 B<EXPERIMENTAL>. Cache API is not yet established.
 
+=item * allow_dots => 1|0 - if true, serve files/directories
+starting with a dot (.git etc), otherwise give a 404.
+
+B<EXPERIMAENTAL>
+
 =item * description - comment. The default is "Static content at $dir"
 
 =back
@@ -332,7 +376,7 @@ This is the intended usage.
 =cut
 
 my %static_options;
-$static_options{$_}++ for qw( description buffer cache_ttl );
+$static_options{$_}++ for qw( description buffer cache_ttl allow_dots );
 sub static {
     my ($self, $path, $dir, %options) = @_;
     $self = $Inst unless ref $self;
@@ -345,6 +389,8 @@ sub static {
     my $xfiles = MVC::Neaf::X::Files->new( %options, root => $dir );
     return $self->route($path => $xfiles->make_handler
         , method => ['GET', 'HEAD']
+        , path_info_regex => '.*'
+        , cache_ttl => $options{cache_ttl}
         , description => $options{description} || "Static content at $dir" );
 };
 
@@ -687,7 +733,9 @@ sub _make_route_re {
     $hash ||= $self->{route};
 
     my $re = join "|", map { quotemeta } reverse sort keys %$hash;
-    return qr{^($re)(/[^?]*)?(?:\?|$)};
+
+    # make $1, $2 always defined
+    return qr{^($re)((?:/[^?]*)?)(?:\?|$)};
 };
 
 =head1 EXPORTED FUNCTIONS
@@ -879,11 +927,18 @@ sub handle_request {
         };
 
         # Lookup the rules for the given path
-        $req->path =~ $self->{route_re} and $route = $self->{route}{$1}
+        $req->path =~ $self->{route_re} and my $node = $self->{route}{$1}
             or die "404\n";
-        $route = $route->{ $req->method }
-            or die "405\n";
-        $req->set_full_path( $1, $2 );
+        unless ($route = $node->{ $req->method }) {
+            $req->set_header( Allow => join ", ", keys %$node );
+            die "405\n";
+        };
+
+        # TODO optimize this or do smth. Still MUST keep route_re a prefix tree
+        my ($path, $path_info_regex) = ($1, $2);
+        $path_info_regex =~ $route->{path_info_regex_regex}
+            or die "404\n";
+        $req->set_full_path( $path, $path_info_regex, !$route->{has_path_info_regex} );
 
         # Run the controller!
         return $route->{code}->($req);
@@ -957,6 +1012,8 @@ sub handle_request {
     $head->push_header( set_cookie => $req->format_cookies );
     $head->init_header( content_length => length $content )
         unless $data->{-continue};
+    $head->init_header( expires => _http_date( time + $route->{cache_ttl} ) )
+        if exists $route->{cache_ttl} and $data->{-status} == 200;
     $content = '' if $req->method eq 'HEAD';
 
     # END PROCESS REPLY
@@ -1019,6 +1076,12 @@ sub _error_to_reply {
     };
 };
 
+# TODO Make this public?.. Move to Neaf::Util?
+sub _http_date {
+    my $t = shift;
+    return strftime( "%a, %d %b %Y %H:%M:%S GMT", gmtime($t))
+};
+
 sub _croak {
     my ($self, $msg) = @_;
 
@@ -1027,6 +1090,29 @@ sub _croak {
     croak( (ref $self || $self)."->$where: $msg" );
 };
 
+=head2 run_test( \%PSGI_ENV )
+
+Run a PSGI request and return ($status, HTTP::Headers, $whole_content ).
+
+Just as the name suggests, useful for testing only (it reduces boilerplate).
+
+Does NOT handle continuation requests.
+
+=cut
+
+sub run_test {
+    my ($self, $env) = @_;
+
+    my $ret = $self->run->( $env );
+    # TODO handle functional return, too!
+
+    return (
+        $ret->[0],
+        HTTP::Headers->new( @{ $ret->[1] } ),
+        join '', @{ $ret->[2] },
+    );
+};
+
 =head1 MORE EXAMPLES
 
 See the examples directory in this distro or at
