@@ -3,7 +3,7 @@ package MVC::Neaf::View::JS;
 use strict;
 use warnings;
 
-our $VERSION = 0.1201;
+our $VERSION = 0.13;
 
 =head1 NAME
 
@@ -24,11 +24,11 @@ instead or rendering a template.
 
 =cut
 
-use JSON::XS;
+use JSON;
 
 use parent qw(MVC::Neaf::View);
 
-my $codec = JSON::XS->new->allow_blessed->convert_blessed->allow_unknown;
+my $codec = JSON->new->allow_blessed->convert_blessed->allow_unknown;
 my $js_id_re = qr/[A-Z_a-z][A-Z_a-z\d]*/;
 my $jsonp_re = qr/^$js_id_re(?:\.$js_id_re)*$/;
 
