@@ -2,7 +2,7 @@ package MVC::Neaf::X::Files;
 
 use strict;
 use warnings;
-our $VERSION = 0.1201;
+our $VERSION = 0.13;
 
 =head1 NAME
 
@@ -29,6 +29,7 @@ So this module is here to fill the gap.
 
 =cut
 
+use POSIX qw(strftime);
 use parent qw(MVC::Neaf::X);
 
 =head2 new( %options )
@@ -89,30 +90,35 @@ sub make_handler {
     my $handler = sub {
         my $req = shift;
 
-        my $file = $req->path_info( '.*' );
-
-        # sanitize file path
-        $file =~ m#/../# and die 404;
-        $file =~ s#^/*#/#;
-        $file =~ s#/*$##;
-        $file =~ s#/+#/#g;
-
+        my $file = $req->path_info();
         my $time = time;
 
+        # sanitize file path before caching
+        $file = "/$file";
+        $file =~ s#/+#/#g;
+        $file =~ s#/$##;
+
         if (my $data = $self->{cache_content}{$file}) {
             if ($data->{expire} < $time) {
                 delete $self->{cache_content}{$file};
             } else {
                 $req->set_header( content_disposition => $data->{disposition} )
                     if $data->{disposition};
+                $data->{expire_head} ||= _http_date( $data->{expire} );
+                $req->set_header( expires => $data->{expire_head} );
                 return { -content => $data->{data}, -type => $data->{type} };
             };
         };
 
+        # don't let unsafe paths through
+        $file =~ m#/../# and die 404;
+        $file =~ m#(^|/)\.# and die 404
+            unless $self->{allow_dots};
+
         # open file
         my $xfile = join "", $dir, $file;
 
-        die 403 if -d $xfile;
+        die 404 if -d $xfile; # Sic! Don't reveal directory structure
         my $ok = open (my $fd, "<", "$xfile");
         if (!$ok) {
             # TODO Warn
@@ -140,10 +146,12 @@ sub make_handler {
         # return whole file if possible
         if ($size < $bufsize) {
             if ($self->{cache_ttl}) {
-                $self->{cache_content}{$file}{data} = $buf;
-                $self->{cache_content}{$file}{expire} = $time + $self->{cache_ttl};
-                $self->{cache_content}{$file}{type} = $type;
-                $self->{cache_content}{$file}{disposition} = $disposition;
+                my %content;
+                $content{data} = $buf;
+                $content{expire} = $time + $self->{cache_ttl};
+                $content{type} = $type;
+                $content{disposition} = $disposition;
+                $self->{cache_content}{$file} = \%content;
             };
             return { -content => $buf, -type => $type }
         };
@@ -167,4 +175,10 @@ sub make_handler {
     return $handler;
 };
 
+# TODO Copied from MVC::Neaf as is. Maybe Neaf::Util?
+sub _http_date {
+    my $t = shift;
+    return strftime( "%a, %d %b %Y %H:%M:%S GMT", gmtime($t))
+};
+
 1;
